@@ -10,6 +10,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdint.h>
 
 #include <zlib.h>
 #include <bzlib.h>
@@ -126,49 +127,46 @@ recode_instr(struct instr *instr, int instrlen, unsigned int **b1p, int *nb1p, u
   unsigned int *b2;
   int nb2;
   int i, j;
-  int outoff, lastoff;
-  unsigned int x1, x2, x4;
-  int x3;
+  int lastoff;
+  unsigned int x1, x2, x3, x4;
   unsigned int left;
   int payp;
 
   b1 = b2 = 0; 
   nb1 = nb2 = 0;
   j = 0;  
-  outoff = lastoff = 0;
+  lastoff = 0;
   left = pay ? pay[0].lx : 0;
   payp = 0;
   for (i = 0; i < instrlen; i++)
     {
       x1 = instr[i].copyout;
       x2 = instr[i].copyin;
-      x3 = instr[i].seekout;
+      x3 = instr[i].copyoutoff;
       x4 = instr[i].copyinoff;
 retry:
       if (!x1)
         {
-          outoff += x3;
-          x3 = 0;
           if (!x2)
             continue;
         }
       if (x1)
         {
           if ((nb2 & 15) == 0)
-            b2 = xrealloc(b2, (nb2 + 16) * 2 * sizeof(unsigned int));
-          if (lastoff < outoff)
-            b2[2 * nb2] = outoff - lastoff;
+            b2 = xrealloc2(b2, nb2 + 16, 2 * sizeof(unsigned int));
+          if (lastoff <= x3)
+            b2[2 * nb2] = x3 - lastoff;
           else
-            b2[2 * nb2] = (lastoff - outoff) | 0x80000000;
+            b2[2 * nb2] = (lastoff - x3) | 0x80000000;
           if (left && x1 >= left)
 	    {
 	      b2[2 * nb2 + 1] = left;
 	      nb2++;
 	      j++;
-	      outoff = lastoff = outoff + left;
+	      x3 = lastoff = x3 + left;
 	      x1 -= left;
 	      if ((nb1 & 15) == 0)
-		b1 = xrealloc(b1, (nb1 + 16) * 3 * sizeof(unsigned int));
+		b1 = xrealloc2(b1, nb1 + 16, 3 * sizeof(unsigned int));
 	      b1[3 * nb1] = j;
 	      b1[3 * nb1 + 1] = 0xffffffff;
 	      b1[3 * nb1 + 2] = payp;
@@ -186,23 +184,17 @@ retry:
 	      b2[2 * nb2 + 1] = x1;
 	      nb2++;
 	      j++;
-	      lastoff = outoff + x1;
-	      outoff = lastoff + x3;
+	      x3 = lastoff = x3 + x1;
 	      if (left)
 	        left -= x1;
 	    }
-          if ((int)outoff < 0)
-            {
-              fprintf(stderr, "ctrlstrm: negative seek\n");
-              exit(1);
-            }
         }
       if (x2)
         {
           if (left && x2 >= left)
 	    {
 	      if ((nb1 & 15) == 0)
-		b1 = xrealloc(b1, (nb1 + 16) * 3 * sizeof(unsigned int));
+		b1 = xrealloc2(b1, nb1 + 16, 3 * sizeof(unsigned int));
 	      b1[3 * nb1] = j;
 	      b1[3 * nb1 + 1] = left;
 	      b1[3 * nb1 + 2] = x4;
@@ -211,7 +203,7 @@ retry:
 	      x2 -= left;
 	      x4 += left;
 	      if ((nb1 & 15) == 0)
-		b1 = xrealloc(b1, (nb1 + 16) * 3 * sizeof(unsigned int));
+		b1 = xrealloc2(b1, nb1 + 16, 3 * sizeof(unsigned int));
 	      b1[3 * nb1] = 0;
 	      b1[3 * nb1 + 1] = 0xffffffff;
 	      b1[3 * nb1 + 2] = payp;
@@ -221,11 +213,11 @@ retry:
 		;
 	      if (payp < payn)
 		left = pay[payp].lx;
-	      x1 = x3 = 0;
+	      x1 = 0;
 	      goto retry;
 	    }
           if ((nb1 & 15) == 0)
-            b1 = xrealloc(b1, (nb1 + 16) * 3 * sizeof(unsigned int));
+            b1 = xrealloc2(b1, nb1 + 16, 3 * sizeof(unsigned int));
           b1[3 * nb1] = j;
           b1[3 * nb1 + 1] = x2;
           b1[3 * nb1 + 2] = x4;
@@ -243,7 +235,7 @@ retry:
   if (j)
     {
       if ((nb1 & 15) == 0)
-        b1 = xrealloc(b1, (nb1 + 16) * 3 * sizeof(unsigned int));
+        b1 = xrealloc2(b1, nb1 + 16, 3 * sizeof(unsigned int));
       b1[3 * nb1] = j;
       b1[3 * nb1 + 1] = 0;
       b1[3 * nb1 + 2] = 0;
@@ -443,7 +435,7 @@ void diffit(struct cfile *fpout, FILE *fpold, FILE *fpnew, unsigned char *old, u
   struct instr *instr = 0;
   int instrlen = 0;
   unsigned char *addblk = 0;
-  int addblklen = 0;
+  unsigned int addblklen = 0;
   int i, j, b2i;
   unsigned int off;
 
